==> nsnake-1.0.1.tgz: nsnake-1.0.1/nsnake.c <==
/*
 * Copyright (c) 2011, David "markand" Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <signal.h>
#include <unistd.h>
#include <curses.h>
#include <time.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/stat.h>

#define SCOREFILE	"/var/db/nsnake/scores"

#define	HEIGHT	23
#define WIDTH	78
#define SIZE	((HEIGHT - 2) * (WIDTH - 2))

#define EMPTY	0
#define WALL	1
#define SNAKE	2
#define FOOD	3

#define WSET(frame, pair)	if (setcolors) wattron(frame, pair)
#define WUNSET(frame, pair)	if (setcolors) wattroff(frame, pair)
#define REFRESH()		refresh(); wrefresh(top); wrefresh(frame);

/* Snake structure */
struct snake {
	short	score;	/* user score */
	size_t	length;	/* snake's size */
	int8_t	dirx;	/* direction in x could be 0, 1 or -1 */
	int8_t	diry;	/* same for y */

	struct {
		uint8_t x;	/* each snake part has (x, y) position */
		uint8_t y;	/* each part will be displayed */
	} pos[SIZE];
};

/* Food structure */
struct food {
	enum {
		NORM = 0,	/* both increase the score but only NORM */
		FREE		/* increase the snake's length too */
	} type;

	uint8_t x;
	uint8_t y;
};

static bool	setcolors = true;	/* enable colors */
static bool	warp = true;		/* enable warp from wall */

static uint8_t	grid[HEIGHT][WIDTH] = { { 0 } };
static WINDOW	*top = NULL;
static WINDOW	*frame = NULL;

static bool	uinit(struct snake *, struct food *);
static void	setgrid(struct snake *);
static void	draw(struct snake *, struct food *);
static bool	isdead(struct snake *);
static bool	iseaten(struct snake *, struct food *);
static void	spawn(struct food *);
static void	direction(struct snake *, struct timespec *, int);
static bool	insertscore(struct snake *);
static void	showscores(void);
static void	resizehandler(int);
static void	quit(void);
static void	usage(void);

int
main(int argc, char *argv[])
{
	bool	running;
	int	x, y, ch;

	struct snake sn = {
		.score = 0,
		.length = 4,
		.dirx = 1,
		.diry = 0,
		.pos = { {5, 10}, {5, 9}, {5, 8}, {5, 7} }
	};

	struct food fd = { .type = NORM };
	struct timespec delay = { .tv_sec = 0, .tv_nsec = 100000000 };

	/* Process options */
	while ((ch = getopt(argc, argv, "cws")) != -1) {
		switch (ch) {
		case 'c': setcolors = 0; break;
		case 'w': warp = false; break;
		case 's': showscores(); return 0;
		case '?':
		default :
			  usage();
		}
	}

	argc -= optind;
	argv += optind;

	/* Game init */
	srandom(time(NULL));
	signal(SIGWINCH, resizehandler);

	if (!uinit(&sn, &fd) != 0) {
		quit();
		errx(1, "Terminal too small, aborting");
	}
		
	if (top == NULL || frame == NULL) {
		endwin();
		errx(1, "ncurses failed to init");
	}

	/* Apply WALL to the edges */
	for (y = 0; y < HEIGHT; ++y)
		grid[y][0] = grid[y][WIDTH - 1] = WALL;
	for (x = 0; x < WIDTH; ++x)
		grid[0][x] = grid[HEIGHT - 1][x] = WALL;

	/* Do not spawn food on snake */
	setgrid(&sn);

	do
		spawn(&fd);
	while (grid[fd.y][fd.x] != EMPTY);

	/* Apply food on the grid */
	grid[fd.y][fd.x] = FOOD;
	draw(&sn, &fd);

	/* First direction is to right */
	sn.pos[0].x += 1;

	box(frame, ACS_VLINE, ACS_HLINE);

	REFRESH();

	running = true;
	while (!isdead(&sn) && running) {
		if (iseaten(&sn, &fd)) {
			if (fd.type == NORM)
				sn.length += 2;

			do
				spawn(&fd);
			while (grid[fd.y][fd.x] != EMPTY);

			sn.score += 1;
			grid[fd.y][fd.x] = FOOD;
		}

		ch = getch();
		if (ch == 'p') {
			nodelay(stdscr, FALSE);
			while ((ch = getch()) != 'p')
				;
			nodelay(stdscr, TRUE);
		}
		else if (ch == 'q')
			running = false;
		else if (ch)
			direction(&sn, &delay, ch);

		draw(&sn, &fd);
		setgrid(&sn);

		sn.pos[0].x += sn.dirx;
		sn.pos[0].y += sn.diry;

		/* If warp enabled, touching wall warps to the opposite */
		if (warp) {
			if (sn.pos[0].x == WIDTH - 1)
				sn.pos[0].x = 1;
			else if (sn.pos[0].x == 0)
				sn.pos[0].x = WIDTH - 2;
			else if (sn.pos[0].y == HEIGHT - 1)
				sn.pos[0].y = 1;
			else if (sn.pos[0].y == 0)
				sn.pos[0].y = HEIGHT - 2;
		}

		nanosleep(&delay, NULL);
	}

	/* The snake is dead. */
	quit();

	/* User has left or is he dead? */
	printf("%sScore: %d\n", (grid[sn.pos[0].y][sn.pos[0].x] != EMPTY) ?
	    "You died...\n" : "", sn.score);

	if (!insertscore(&sn))
		err(1, "Could not write score file %s", SCOREFILE);
	
	return 0;
}

static bool
uinit(struct snake *sn, struct food *fd)
{
	/* Ncurses init */
	initscr(); noecho(); curs_set(FALSE);
	keypad(stdscr, TRUE); nodelay(stdscr, TRUE);

	if (COLS < (WIDTH + 1) || LINES < (HEIGHT + 1))	
		return false;

	if (setcolors == 1 && has_colors()) {
		use_default_colors();
		start_color();
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_GREEN, -1);
		init_pair(3, COLOR_YELLOW, -1);
	}

	top = newwin(1, 0, 0, 0);

	if (setcolors) {
		wbkgd(top, COLOR_PAIR(1));
		wattrset(top, COLOR_PAIR(1) | A_BOLD);
	}

	frame = newwin(HEIGHT, WIDTH, (LINES/2)-(HEIGHT/2), (COLS/2)-(WIDTH/2));
	REFRESH();

	return true;
}

static void
setgrid(struct snake *sn)
{
	size_t i;
	for (i = 0; i < sn->length; ++i)
		grid[sn->pos[i].y][sn->pos[i].x] = SNAKE;

	/*
	 * each snake part must follow the last part, pos[0] is head, then
	 * pos[2] takes pos[1] place, pos[3] takes pos[2] and so on.
	 */
	grid[sn->pos[sn->length-1].y][sn->pos[sn->length-1].x] = EMPTY;
	memmove(&sn->pos[1], &sn->pos[0], sizeof (sn->pos));
}

static void
draw(struct snake *sn, struct food *fd)
{
	size_t i;

	for (i = 0; i < sn->length; ++i) {
		WSET(frame, COLOR_PAIR(2));
		(void) mvwaddch(frame, sn->pos[i].y, sn->pos[i].x, '#');
		WUNSET(frame, COLOR_PAIR(2));
	}

	/* Print head */
	WSET(frame, COLOR_PAIR(2) | A_BOLD);
	(void) mvwaddch(frame, sn->pos[0].y, sn->pos[0].x, '@');
	WUNSET(frame, COLOR_PAIR(2) | A_BOLD);

	/* Erase the snake's tail */
	(void) mvwaddch(frame, sn->pos[sn->length].y, sn->pos[sn->length].x, ' ');

	/* Print food */
	WSET(frame, COLOR_PAIR(3) | A_BOLD);
	(void) mvwaddch(frame, fd->y, fd->x, (fd->type == FREE) ? '*' : '+');
	WUNSET(frame, COLOR_PAIR(3) | A_BOLD);

	/* Print score */
	(void) wmove(top, 0, 0);
	(void) wprintw(top, "Score : %d", sn->score);

	REFRESH();
}

static bool
isdead(struct snake *sn)
{
	if (grid[sn->pos[0].y][sn->pos[0].x] == SNAKE)
		return true;

	/* No warp enabled means dead in wall */
	if (!warp)
		if (grid[sn->pos[0].y][sn->pos[0].x] == WALL)
			return true;

	/* Too big for the screen */
	if (sn->length >= SIZE)
		return true;

	return false;
}

static bool
iseaten(struct snake *sn, struct food *fd)
{
	if (grid[sn->pos[0].y][sn->pos[0].x] == FOOD)
		return true;

	return false;
}

static void
spawn(struct food *fd)
{
	int num;

	fd->x = (random() % (WIDTH - 2)) + 1;
	fd->y = (random() % (HEIGHT - 2)) + 1;

	num = ((random() % 7) - 1) + 1;

	/* This is a free food (does not shrink) */
	if (num == 6)
		fd->type = FREE;
	else
		fd->type = NORM;
}

static void
direction(struct snake *sn, struct timespec *tm, int ch)
{
	switch (ch) {
	case KEY_LEFT: case 'h': case 'H':
		if (sn->dirx != 1) { sn->dirx = -1; sn->diry = 0; }
		break;
	case KEY_UP: case 'k': case 'K':
		if (sn->diry != 1) { sn->dirx = 0; sn->diry = -1; }
		break;
	case KEY_DOWN: case 'j': case 'J':
		if (sn->diry != -1) { sn->dirx = 0; sn->diry = 1; }
		break;
	case KEY_RIGHT: case 'l': case 'L':
		if (sn->dirx != -1) { sn->dirx = 1; sn->diry = 0; }
	default:
		break;
	}
}

static bool
insertscore(struct snake *sn)
{
	struct stat	st;
	struct tm	*tm;

	FILE		*fp;
	char		*buf, *pn, str[100], nick[25];
	off_t		prev = 0, next = 0, pos = 0;
	short		high;
	size_t		length;
	time_t		now;

	/* Save the current date in dd,mm,yyyy:hh:mm format */
	now = time(NULL);
	tm = localtime(&now);

	/* File does not exists, create it and write score */
	if (access(SCOREFILE, F_OK) != 0) {
		if ((fp = fopen(SCOREFILE, "w+")) == NULL)
			return false;

		fprintf(fp, "%d:%s:%d,%d,%d,%d,%d\n", sn->score,
		    getpwuid(getuid())->pw_name, tm->tm_mday, tm->tm_mon,
		    tm->tm_year, tm->tm_hour, tm->tm_min);
		return true;
	}

	/* Get file size and open a file pointer */
	if (stat(SCOREFILE, &st) != 0 || (fp = fopen(SCOREFILE, "r")) == NULL)
		return false;

	/* Store the current file content in file */
	if ((buf = calloc(st.st_size + 1, 1)) == NULL)
		return false;

	fread(buf, 1, st.st_size, fp);

	/* First init */
	pn = strchr(buf, '\n');
	sscanf(buf, "%hd:%[^:]", &high, nick);

	/* Per line analysis */
	while (sn->score < high && (pn = strchr(&buf[next + 1], '\n')) != NULL) {
		sscanf(&buf[next], "%hd:%[^:]", &high, nick);
		prev = next;
		next += pn - (buf + next) + 1;
	}

	if (strcmp(nick, getpwuid(getuid())->pw_name) == 0 && sn->score == high)
		return true;

	/* This is the user nick to save */
	strncpy(nick, getpwuid(getuid())->pw_name, sizeof (nick));

	/* If end of file append to the end, if not append to the previous */
	pos = (pn == NULL) ? next : prev;

	/* Alloc enough to store the new line including \n */
	sprintf(str, "%d:%s:%d,%d,%d,%d,%d", sn->score, nick, tm->tm_mday,
	    tm->tm_mon, tm->tm_year, tm->tm_hour, tm->tm_min);
	length = strlen(str) + 1;	
	buf = realloc(buf, st.st_size + length);

	/* Move to insert the new string */
	memmove(&buf[pos + length], &buf[pos], st.st_size - pos);
	strncpy(&buf[pos], str, length);

	/* Remove the \0 and replace it by \n */
	buf[pos + length - 1] = '\n';

	if ((fp = freopen(SCOREFILE, "w", fp)) == NULL)
		return false;

	/* Finally write the new file */
	fwrite(buf, st.st_size + length, 1, fp);

	free(buf);
	fclose(fp);

	return true;
}

static void
showscores(void)
{
	struct tm	tm;
	FILE		*fp;
	short		score;
	char		buf[100], toprint[100];
	char		name[25];

	if ((fp = fopen(SCOREFILE, "r")) == NULL)
		err(1, "Could not read %s", SCOREFILE);

	while (fgets(buf, sizeof (buf), fp) != NULL) {
		/* Analyse the date format */
		memset(name, '\0', sizeof (name));
		sscanf(buf, "%hd:%[^:]:%d,%d,%d,%d,%d", &score, name,
		    &tm.tm_mday, &tm.tm_mon, &tm.tm_year, &tm.tm_hour,
		    &tm.tm_min);
	
		strftime(toprint, sizeof (toprint), "%d/%m %Y %H:%M", &tm);
		printf("%s has died with %d point%s the %s\n", name, score,
		    (score > 1) ? "s" : "", toprint);
	}
}

void
resizehandler(int signal)
{
	int x, y;

	if (signal != SIGWINCH)
		return;

	/* XXX: I would like to pause the game until the terminal is resized */
	endwin(); resizeterm(LINES, COLS);
	refresh(); clear();

	/* Color the top bar */
	wbkgd(top, COLOR_PAIR(1) | A_BOLD);

	getmaxyx(stdscr, y, x);

	if (x < WIDTH || y < HEIGHT) {
		quit();
		errx(1, "Terminal has been resized too small, aborting");
	}

	mvwin(frame, (y / 2) - (HEIGHT / 2), (x / 2) - (WIDTH / 2));

	REFRESH();
}

static void
quit(void)
{
	delwin(top); delwin(frame); delwin(stdscr);
	endwin();
}

static void
usage(void)
{
	fprintf(stderr, "usage: nsnake [-cwh]\n");
	exit(1);
}

==> nsnake-1.1.tgz: nsnake-1.1/nsnake.c <==
/*
 * Copyright (c) 2011, David "markand" Demelier <markand@malikania.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <signal.h>
#include <unistd.h>
#include <curses.h>
#include <time.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/stat.h>

#define SCOREFILE	"/var/db/nsnake/scores"

#define	HEIGHT		23
#define WIDTH		78
#define SIZE		((HEIGHT - 2) * (WIDTH - 2))

#define EMPTY		0
#define WALL		1
#define SNAKE		2
#define FOOD		3

#define WSET(frame, pair)	if (setcolors) wattron(frame, pair)
#define WUNSET(frame, pair)	if (setcolors) wattroff(frame, pair)
#define REFRESH()		refresh(); wrefresh(top); wrefresh(frame);

/* Snake structure */
struct snake {
	short	score;		/* user score */
	size_t	length;		/* snake's size */
	int8_t	dirx;		/* direction in x could be 0, 1 or -1 */
	int8_t	diry;		/* same for y */

	struct {
		uint8_t x;	/* each snake part has (x, y) position */
		uint8_t y;	/* each part will be displayed */
	} pos[SIZE];
};

/* Food structure */
struct food {
	enum {
		NORM = 0,	/* both increase the score but only NORM */
		FREE		/* increase the snake's length too */
	} type;

	uint8_t x;		/* Position of the current food, will be used */
	uint8_t y;		/* in grid[][]. */
};

/* General options */
static bool	setcolors = true;	/* enable colors */
static bool	warp = true;		/* enable wall crossing */

/* Global variables */
static uint8_t	grid[HEIGHT][WIDTH] = { { EMPTY } };
static WINDOW	*top = NULL;
static WINDOW	*frame = NULL;

static bool	uinit(struct snake *, struct food *);
static void	setgrid(struct snake *);
static void	draw(struct snake *, struct food *);
static bool	isdead(struct snake *);
static bool	iseaten(struct snake *, struct food *);
static void	spawn(struct food *);
static void	direction(struct snake *, struct timespec *, int);
static bool	insertscore(struct snake *);
static void	showscores(void);
static void	resizehandler(int);
static void	quit(void);
static void	usage(void);

int
main(int argc, char *argv[])
{
	bool	running;
	int	x, y, ch;

	struct snake sn = {
		.score = 0,	.length = 4,
		.dirx = 1,	.diry = 0,
		.pos = { {5, 10}, {5, 9}, {5, 8}, {5, 7} }
	};

	struct food fd = { .type = NORM };
	struct timespec delay = { .tv_sec = 0, .tv_nsec = 100000000 };

	/* Process options */
	while ((ch = getopt(argc, argv, "cws")) != -1) {
		switch (ch) {
		case 'c': setcolors = false; break;
		case 'w': warp = false; break;
		case 's': showscores(); return 0;
		case '?':
		default :
			  usage();
		}
	}

	argc -= optind;
	argv += optind;

	/* Game init */
	srandom(time(NULL));
	signal(SIGWINCH, resizehandler);

	if (!uinit(&sn, &fd) != 0) {
		quit();
		errx(1, "Terminal too small, aborting");
	}
		
	if (top == NULL || frame == NULL) {
		endwin();
		errx(1, "ncurses failed to init");
	}

	/* Apply WALL to the edges */
	for (y = 0; y < HEIGHT; ++y)
		grid[y][0] = grid[y][WIDTH - 1] = WALL;
	for (x = 0; x < WIDTH; ++x)
		grid[0][x] = grid[HEIGHT - 1][x] = WALL;

	/* Do not spawn food on snake */
	setgrid(&sn);
	spawn(&fd);

	/* Apply food on the grid */
	grid[fd.y][fd.x] = FOOD;
	draw(&sn, &fd);

	/* First direction is to right */
	sn.pos[0].x += sn.dirx;

	box(frame, ACS_VLINE, ACS_HLINE);

	REFRESH();

	running = true;
	while (!isdead(&sn) && running) {
		/* Draw and define grid with snake parts */
		draw(&sn, &fd);
		setgrid(&sn);

		/* Go to the next position */
		sn.pos[0].x += sn.dirx;
		sn.pos[0].y += sn.diry;

		if (iseaten(&sn, &fd)) {
			if (fd.type == NORM)
				sn.length += 2;

			/* If the screen is totally filled */
			if (sn.length == SIZE) {
				int i = 0;

				/* Emulate new game */
				for (i = 4; i < SIZE; ++i) {
					(void) mvwaddch(frame, sn.pos[i].y, sn.pos[i].x, ' ');
					grid[sn.pos[i].y][sn.pos[i].x] = EMPTY;
				}

				sn.length = 4;
			}

			spawn(&fd);

			sn.score += 1;
			grid[fd.y][fd.x] = FOOD;
		}

		ch = getch();
		if (ch == 'p') {
			nodelay(stdscr, false);
			while ((ch = getch()) != 'p' && ch != 'q')
				;

			if (ch == 'q')
				running = false;

			nodelay(stdscr, true);
		} else if (ch == 'q')
			running = false;
		else if (ch)
			direction(&sn, &delay, ch);

		/* If warp enabled, touching wall cross to the opposite */
		if (warp) {
			if (sn.pos[0].x == WIDTH - 1)
				sn.pos[0].x = 1;
			else if (sn.pos[0].x == 0)
				sn.pos[0].x = WIDTH - 2;
			else if (sn.pos[0].y == HEIGHT - 1)
				sn.pos[0].y = 1;
			else if (sn.pos[0].y == 0)
				sn.pos[0].y = HEIGHT - 2;
		}

		nanosleep(&delay, NULL);
	}

	/* The snake is dead. */
	quit();

	/* User has left or is he dead? */
	printf("%sScore: %d\n", (grid[sn.pos[0].y][sn.pos[0].x] != EMPTY) ?
	    "You died...\n" : "", sn.score);

	if (!insertscore(&sn))
		err(1, "Could not write score file %s", SCOREFILE);
	
	return 0;
}

static bool
uinit(struct snake *sn, struct food *fd)
{
	/* Ncurses init */
	initscr(); noecho(); curs_set(false);
	keypad(stdscr, true); nodelay(stdscr, true);

	if (COLS < (WIDTH + 1) || LINES < (HEIGHT + 1))	
		return false;

	if (setcolors && has_colors()) {
		use_default_colors();
		start_color();
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_GREEN, -1);
		init_pair(3, COLOR_YELLOW, -1);
	}

	top = newwin(1, 0, 0, 0);

	if (setcolors) {
		wbkgd(top, COLOR_PAIR(1));
		wattrset(top, COLOR_PAIR(1) | A_BOLD);
	}

	frame = newwin(HEIGHT, WIDTH, (LINES/2)-(HEIGHT/2), (COLS/2)-(WIDTH/2));
	REFRESH();

	return true;
}

static void
setgrid(struct snake *sn)
{
	size_t i;

	for (i = 0; i < sn->length; ++i)
		grid[sn->pos[i].y][sn->pos[i].x] = SNAKE;

	/*
	 * each snake part must follow the last part, pos[0] is head, then
	 * pos[2] takes pos[1] place, pos[3] takes pos[2] and so on.
	 */
	grid[sn->pos[sn->length-1].y][sn->pos[sn->length-1].x] = EMPTY;
	memmove(&sn->pos[1], &sn->pos[0], sizeof (sn->pos));
}

static void
draw(struct snake *sn, struct food *fd)
{
	size_t i;

	for (i = 0; i < sn->length; ++i) {
		WSET(frame, COLOR_PAIR(2));
		(void) mvwaddch(frame, sn->pos[i].y, sn->pos[i].x, '#');
		WUNSET(frame, COLOR_PAIR(2));
	}

	/* Print head */
	WSET(frame, COLOR_PAIR(2) | A_BOLD);
	(void) mvwaddch(frame, sn->pos[0].y, sn->pos[0].x, '@');
	WUNSET(frame, COLOR_PAIR(2) | A_BOLD);

	/* Erase the snake's tail */
	(void) mvwaddch(frame, sn->pos[sn->length].y, sn->pos[sn->length].x, ' ');

	/* Print food */
	WSET(frame, COLOR_PAIR(3) | A_BOLD);
	(void) mvwaddch(frame, fd->y, fd->x, (fd->type == FREE) ? '*' : '+');
	WUNSET(frame, COLOR_PAIR(3) | A_BOLD);

	/* Print score */
	(void) wmove(top, 0, 0);
	(void) wprintw(top, "Score : %d", sn->score);

	REFRESH();
}

static bool
isdead(struct snake *sn)
{
	if (grid[sn->pos[0].y][sn->pos[0].x] == SNAKE)
		return true;

	/* No warp enabled means dead in wall */
	if (!warp)
		if (grid[sn->pos[0].y][sn->pos[0].x] == WALL)
			return true;

	return false;
}

static bool
iseaten(struct snake *sn, struct food *fd)
{
	if (grid[sn->pos[0].y][sn->pos[0].x] == FOOD)
		return true;

	return false;
}

static void
spawn(struct food *fd)
{
	int num;

	do {
		fd->x = (random() % (WIDTH - 2)) + 1;
		fd->y = (random() % (HEIGHT - 2)) + 1;
	} while (grid[fd->y][fd->x] != EMPTY);

	num = ((random() % 7) - 1) + 1;

	/* This is a free food (does not shrink) */
	if (num == 6)
		fd->type = FREE;
	else
		fd->type = NORM;
}

static void
direction(struct snake *sn, struct timespec *tm, int ch)
{
	switch (ch) {
	case KEY_LEFT: case 'h': case 'H':
		if (sn->dirx != 1) { sn->dirx = -1; sn->diry = 0; }
		break;
	case KEY_UP: case 'k': case 'K':
		if (sn->diry != 1) { sn->dirx = 0; sn->diry = -1; }
		break;
	case KEY_DOWN: case 'j': case 'J':
		if (sn->diry != -1) { sn->dirx = 0; sn->diry = 1; }
		break;
	case KEY_RIGHT: case 'l': case 'L':
		if (sn->dirx != -1) { sn->dirx = 1; sn->diry = 0; }
	default:
		break;
	}
}

static bool
insertscore(struct snake *sn)
{
	struct stat	st;
	struct tm	*tm;

	FILE		*fp;
	char		*buf, *pn, str[128] = {0}, nick[32 + 1] = {0};
	char		tok;
	off_t		prev = 0, next = 0, pos = 0;
	short		high;
	size_t		length;
	time_t		now;

	/* Save the current date in dd,mm,yyyy:hh:mm format */
	now = time(NULL);
	tm = localtime(&now);

	/* File does not exists, create it and write score */
	if (access(SCOREFILE, F_OK) != 0) {
		if ((fp = fopen(SCOREFILE, "w+")) == NULL)
			return false;

		fprintf(fp, "%c:%d:%s:%d,%d,%d,%d,%d\n", (warp) ? 'w':'n',
		    sn->score, getpwuid(getuid())->pw_name, tm->tm_mday,
		    tm->tm_mon, tm->tm_year, tm->tm_hour, tm->tm_min);

		fclose(fp);

		return true;
	}

	/* Get file size and open a file pointer */
	if (stat(SCOREFILE, &st) != 0 || (fp = fopen(SCOREFILE, "r")) == NULL)
		return false;

	/* Store the current file content in file */
	if ((buf = calloc(st.st_size + 1, 1)) == NULL)
		return false;

	fread(buf, 1, st.st_size, fp);

	/* First init */
	pn = strchr(buf, '\n');
	sscanf(buf, "%c:%hd:%32[^:]", &tok, &high, nick);

	/* Per line analysis */
	while (sn->score < high && (pn = strchr(&buf[next + 1], '\n')) != NULL) {
		if (sscanf(&buf[next], "%c:%hd:%32[^:]", &tok, &high, nick) != 3) {
			fprintf(stderr, "Corrupt score file, aborting\n");
			return true;
		}

		prev = next;
		next += pn - (buf + next) + 1;
	}

	if (strcmp(nick, getpwuid(getuid())->pw_name) == 0 && sn->score == high
	    && tok == ((warp) ? 'w' : 'n'))
		return true;

	/* This is the user nick to save */
	strncpy(nick, getpwuid(getuid())->pw_name, sizeof (nick));

	/* If end of file append to the end, if not append to the previous */
	pos = (pn == NULL) ? next : prev;

	/* Alloc enough to store the new line including \n */
	sprintf(str, "%c:%d:%s:%d,%d,%d,%d,%d", (warp) ? 'w' : 'n', sn->score,
	    nick, tm->tm_mday, tm->tm_mon, tm->tm_year, tm->tm_hour, tm->tm_min);

	length = strlen(str) + 1;	
	buf = realloc(buf, st.st_size + length);

	/* Move to insert the new string */
	memmove(&buf[pos + length], &buf[pos], st.st_size - pos);
	strncpy(&buf[pos], str, length);

	/* Remove the \0 and replace it with \n */
	buf[pos + length - 1] = '\n';

	if ((fp = freopen(SCOREFILE, "w", fp)) == NULL)
		return false;

	/* Finally write the new file */
	fwrite(buf, st.st_size + length, 1, fp);

	free(buf);
	fclose(fp);

	return true;
}

static void
showscores(void)
{
	struct tm	tm;
	FILE		*fp;
	short		score;
	char		buf[128], date[128];
	char		name[32];
	char		tok;

	printf("Wall crossing %s\n", (warp) ? "enabled" : "disabled");

	if ((fp = fopen(SCOREFILE, "r")) == NULL)
		err(1, "Could not read %s", SCOREFILE);

	printf("Date\t\tUser\t\tDate\n");
	while (fgets(buf, sizeof (buf), fp) != NULL) {
		/* Analyse the date format */
		memset(name, '\0', sizeof (name));
		sscanf(buf, "%c:%hd:%[^:]:%d,%d,%d,%d,%d", &tok, &score, name,
		    &tm.tm_mday, &tm.tm_mon, &tm.tm_year, &tm.tm_hour,
		    &tm.tm_min);
	
		if (tok == ((warp) ? 'w' : 'n')) {
			strftime(date, sizeof (date), "%d/%m %Y %H:%M", &tm);
			printf("%d\t\t%s\t\t%s\n", score, name, date);
		}
	}
}

void
resizehandler(int signal)
{
	int x, y;

	if (signal != SIGWINCH)
		return;

	/* XXX: I would like to pause the game until the terminal is resized */
	endwin(); resizeterm(LINES, COLS);
	refresh(); clear();

	/* Color the top bar */
	wbkgd(top, COLOR_PAIR(1) | A_BOLD);

	getmaxyx(stdscr, y, x);

	if (x < WIDTH || y < HEIGHT) {
		quit();
		errx(1, "Terminal has been resized too small, aborting");
	}

	mvwin(frame, (y / 2) - (HEIGHT / 2), (x / 2) - (WIDTH / 2));

	REFRESH();
}

static void
quit(void)
{
	delwin(top); delwin(frame); delwin(stdscr);
	endwin();
}

static void
usage(void)
{
	fprintf(stderr, "usage: nsnake [-cw] [-s]\n");
	exit(1);
}
